;------------------------------------------------------------------------------
; Fu Kung! Copyright (C)2003 Andrew Davie
; see http://www.taswegian.com/TwoHeaded/Atari2600/fukung.html << SOON

; The genesis of the large-sprite system contained herein comes from Eckhard Stohlberg's
; fine work on the 'BIGTWO' sprite system (see the [stella] list archives at
; http://www.biglist.com/lists/stella/archives/ ) and in his preliminary fighter code
; kindly loaned to assist in creating this program.

; The sprite system this program uses is somewhat different to Eckhard's - being more
; efficient and capable (in terms of sprite display) but does essentially retain the
; positioning and movement system of the initial system.  Many thanks ES!

; This code is for a 32K cartridge using the $3F bank-switching scheme.  If running
; with Z26 emulator (v1.57+) use -g11 switch on command-line.



FRAC_SEC        = 5994
TICK            = 10000                    ; FRAC_SEC / TICK = Hz

;------------------------------------------------------------------------------

                processor 6502
                include "vcs3f.h"

DEBUG           = 0         ; non-zero for debug stuff

BGCOL           = $0F
PFCOL           = 0

D0              = 1
D1              = 2
D2              = 4
D3              = 8
D4              = 16
D5              = 32
D6              = 64
D7              = 128

BANK            = $3F       ; write address to switch banks
BANKPAGE        = $F000     ; base address of bank blocks

INITBANK        = 14
BANK_FRAMETABLE = 14
BANK_STATE      = 14
BANK_ANIMATE    = 14

;------------------------------------------------------------------------------
; Code automatically compiles to implement the sprites in the size/format
; specified here.  Can handle up to 5 rows.  


FRAMEBANK00     = 0
FRAMEBANK01     = 1
FRAMEBANK02     = 2
FRAMEBANK03     = 4
FRAMEBANK04     = 5
FRAMEBANK05     = 6
FRAMEBANK06     = 7
FRAMEBANK07     = 8
FRAMEBANK08     = 9
FRAMEBANK09     = 10
FRAMEBANK10     = 11
FRAMEBANK11     = 12
FRAMEBANK12     = 13

DOUBLESIZE      = 0         ; gives small sprites when 0, large when 1
BIG_ROWS        = 6         ; Can't exceed 5 without changing frame table format
BIG_COLS        = 6         ; DO NOT CHANGE

BIG_SPRITES     = 2         ; #creatures

;------------------------------------------------------------------------------
; MACRO definitions

    MAC SETBANK
        lda #{1}
        sta BANK
    ENDM

    MAC ANIMATION
            lda #<{1}
            sta ObjAnimLO,x
            lda #>{1}
            sta ObjAnimHI,x
            lda #1
            sta ObjDelay,x
    ENDM

    MAC HANDLER
            ldy #{1}*2
            jsr SetHandler
    ENDM

;------------------------------------------------------------------------------

                SEG.U variables
                ORG $80


BaseAlpha       ds 1
Phase           ds 1
DrawX           ds 1
DrawY           ds 1

TempCol         ds 2
TempXX          ds 1

LoopCount       ds BIG_ROWS
SPBASE          ds BIG_ROWS * BIG_COLS * 2

BGColour        ds 1

DelayPTR        ds 2

Temp            ds 2

Base            ds 1



;----
ObjFrame        ds BIG_SPRITES
;----


AXPosition      ds BIG_SPRITES
AYPosition      ds BIG_SPRITES

AFacing         ds BIG_SPRITES

                ; F-------
                ; F = facing direction (0 = right, 1 = left)
                ; - = unused


ObjDelay        ds BIG_SPRITES              ; delay count for frame animation
ObjAnimLO       ds BIG_SPRITES              ; animation pointer lo byte
ObjAnimHI       ds BIG_SPRITES              ; animation pointer hi byte
objAnimBank     ds BIG_SPRITES              ; bank containing animation

    ; Note: Possibly include bank with animation hi byte

ObjStateLO      ds BIG_SPRITES              ; state table lo byte
ObjStateHI      ds BIG_SPRITES              ; state table hi byte

ObjFSMLO        ds BIG_SPRITES              ; FSM system lo byte
ObjFSMHI        ds BIG_SPRITES              ; FSM system hi byte


Player          ds 1

JumpPTR         ds 2
FrameAddress    ds 2
Anim            ds 2




TIME_HourAdvance    ds 1
TIME_subSecond  ds 2
TIME_Second     ds 1
TIME_Minute     ds 1
TIME_Hour       ds 1

HMS             ds 4

;------------------------------------------------------------------------------

        ; This overlay variable is used for the overlay variables.  That's OK.
        ; However, it is positioned at the END of the variables so, if on the off chance we're overlapping
        ; stack space and variable, it is LIKELY that that won't be a problem, as the temp variables
        ; (especially the latter ones) are only used in rare occasions.

        ; FOR SAFETY, DO NOT USE THIS AREA DIRECTLY (ie: NEVER reference 'Overlay' in the code)
        ; ADD AN OVERLAY FOR EACH ROUTINE'S USE, SO CLASHES CAN BE EASILY CHECKED

Overlay         ds 0 ;010       ;--> overlay (share) variables


;------------------------------------------------------------------------------
; OVERLAYS!
; These variables are overlays, and should be managed with care
; That is, variables are ALREADY DEFINED, and we're reusing RAM for other purposes

; EACH OF THESE ARE VARIABLES (TEMPORARY) USED BY ONE ROUTINE (AND IT'S SUBROUTINES)
; THAT IS, LOCAL VARIABLES.  USE 'EM FREELY, THEY COST NOTHING

; TOTAL SPACE USED BY ANY OVERLAY GROUP SHOULD BE <= SIZE OF 'Overlay'

;------------------------------------------------------------------------------

                org Overlay

    ; ANIMATION/LOGIC SYSTEM


;------------------------------------------------------------------------------

                org Overlay

    ; DRAWING SYSTEM


;------------------------------------------------------------------------------

            SEG
            ORG $0000
            RORG $f000

    ; GRAPHICS DATA LOADS HERE
    ; WE WILL HAVE MULTIPLE 2K BANKS, EACH USING AN INCLUDE FILE OF GENERATED DATA

            include "bank00.asm"


            SEG
            ORG $0800
            RORG $f000
            include "bank01.asm"

            SEG
            ORG $1000
            RORG $f000
            include "bank02.asm"
        
    ; << bank3 reserved for cuttle-compatibility


            SEG
            ORG $2000
            RORG $f000
            include "bank03.asm"

            SEG
            ORG $2800
            RORG $f000
            include "bank04.asm"

            SEG
            ORG $3000
            RORG $f000
            include "bank05.asm"

            SEG
            ORG $3800
            RORG $f000
            include "bank06.asm"

            SEG
            ORG $4000
            RORG $f000
            include "bank07.asm"

            SEG
            ORG $4800
            RORG $f000
            include "bank08.asm"

            SEG
            ORG $5000
            RORG $f000
            include "bank09.asm"

            SEG
            ORG $5800
            RORG $f000
            include "bank10.asm"

            SEG
            ORG $6000
            RORG $f000
            include "bank11.asm"

            SEG
            ORG $6800
            RORG $f000
            include "bank12.asm"

;------------------------------------------------------------------------------

            SEG
            ORG $7000           ; 2K, ie: bank 1
            RORG $f000          ; lives at $F000


InitX
.XP     SET 90
    REPEAT BIG_SPRITES
    .byte .XP
.XP     SET .XP - 50
    REPEND


    ; There's no particular reason why this code lives in a separate bank
    ; but it's as good-a-test as any for checking the bankswitching is working


Cart_Init
            sei
            cld
               
		    ldx #$FF
		    txs 				; Clear the stack

            lda #0
            sta SWBCNT          ; console I/O always set to INPUT
            sta SWACNT          ; set controller I/O to INPUT

GameInit



    ; Initialise creatures

            ldx #BIG_SPRITES-1
PreClear    

            lda #0
            sta AFacing,x

            lda #100
            sta AYPosition,x

            lda InitX,x
            sta AXPosition,x

            SETBANK BANK_STATE
            
            ldy #STATE_NORMAL
            jsr SetState
            jsr ProcessFSM              ; give creature a chance to init

            SETBANK BANK_ANIMATE
            jsr AnimateObject               ; process object animation sequence

            dex
            bpl PreClear

    ;----------------------------------
    ; Hardwire init a single creature...

            lda #D7
            sta AFacing+1

            ldx #0
            HANDLER NORMAL_STAND
  

    ;----------------------------------


            lda #BGCOL
            sta BGColour

            jmp NewFrame



    include "bank_FRAMETABLE.asm"


;------------------------------------------------------------------------------
; OBJECT Animation
; *MUST* BE IN ONE BANK

NOFRAME     equ $FF



AnimateObject

            lda ObjAnimHI,x
            beq noAnim                      ; animation switched off - use a null frame

            lda ObjDelay,x
            beq exitAnim                    ; static frame - no animation
            dec ObjDelay,x
            bne exitAnim                    ; not ready to change

ReAnimate   lda ObjAnimHI,x
            sta Anim+1
            lda ObjAnimLO,x
            sta Anim                        ; (Anim) points to frame sequence

            ldy #0
            lda (Anim),y
            bpl NormalFrame

                ; Handle a JUMP

            iny
            lda (Anim),y
            sta ObjAnimLO,x
            iny
            lda (Anim),y
            sta ObjAnimHI,x
            jmp ReAnimate

NormalFrame sta ObjFrame,x

            iny
            lda (Anim),y
            sta ObjDelay,x                  ; should be < 128

            clc
            lda Anim
            adc #2
            sta ObjAnimLO,x
            lda Anim+1
            adc #0
            sta ObjAnimHI,x

            rts
            
noAnim      lda #NOFRAME
            sta ObjFrame,x

exitAnim    rts


JUMP        equ 128

SampleAnimation

            .byte 1,2
            .byte 2,2
            .byte 3,1
            .byte 4,1
            .byte 5,1
            .byte 6,5
            .byte 7,5
            .byte 8,1
            .byte 9,1
            .byte 10,1
            .byte 11,1
            .byte 12,5
            
            .byte JUMP
            .word SampleAnimation


ANIMATION_STAND

    ; a sort of in-place dance

;            .byte 4,5
;            .byte 5,5
            .byte 6,5
            .byte 7,5
            .byte JUMP
            .word ANIMATION_STAND


; OBJECT Animation END
;------------------------------------------------------------------------------

;******************************************************************************

;------------------------------------------------------------------------------
; STATE Processing
; *MUST* BE IN ONE BANK




SetState

    ; Set object's state base
    ; x = obj
    ; y = state table (*2)

            lda StateTables,y
            sta ObjStateLO,x
            lda StateTables+1,y
            sta ObjStateHI,x

            ldy #DEFAULT                        ; fall through and include DEFAULT init for handler

SetHandler

    ; Change object's processing code
    ; x = obj
    ; y = new handler entry (word access)

            lda ObjStateHI,x
            beq NullHandler                     ; don't handle anything with no entry

            sta Temp+1
            lda ObjStateLO,x
            sta Temp

            lda (Temp),y
            sta ObjFSMLO,x
            iny
            lda (Temp),y
            sta ObjFSMHI,x

NullHandler rts
   

ProcessFSM

    ; Vector to the creature's processing code
    ; x = creature

            lda ObjFSMLO,x
            sta Temp
            lda ObjFSMHI,x
            sta Temp+1
            jmp (Temp)



DEFAULT         = 0                     ; default entry when switching states


    ;------------------------------------------------------------------------------

STATE_NORMAL    = 0

StateTables

    ; Table of pointers to state tables

            .word StateNormal               ; 0     STATE_NORMAL

    ;------------------------------------------------------------------------------
    ; NORMAL state table
    ; State table is a table of pointers to code processing 'EVENTS'

StateNormal

NORMAL_DEFAULT  = 0
NORMAL_STAND    = 1
NORMAL_STAND2   = 2

            .word NormalDefault             ; 0     DEFAULT
            .word NormalStand               ; 1     NORMAL_STAND
            .word NormalStand2              ; 2     NORMAL_STAND2

    ;------------------------------------------------------------------------------

NormalDefault

            ANIMATION ANIMATION_STAND
            HANDLER NORMAL_STAND2
            rts


NormalStand

            ANIMATION SampleAnimation
            HANDLER NORMAL_STAND2
            rts

NormalStand2
            rts



; STATE Processing END
;------------------------------------------------------------------------------

;******************************************************************************



            SEG bank31
            ORG $7800
            RORG $f800

VBProcessing

    ;------------------------------------------------------------------------


            ldy Player
            iny
            cpy #BIG_SPRITES
            bcc POK
            ldy #0
POK         sty Player
      
            lda AXPosition,y
            sta DrawX
            lda AYPosition,y
            sta DrawY

    ; Calculate the frame definition start address.  Each frame is a 32-byte
    ; definition block giving the bank containing the frame and a matrix of
    ; 16-sprite numbers making up the nx6 grid.

    ; FORMAT:
    ;    BYTE       colour                  tentative colour of frame
    ;    BYTES...    0, 1, 2, 3, 4, 5   \
    ;                6, 7, 8, 9,10,11    |  n x 6 matrix of 16-sprite #s
    ;                ....                |
    ;               24,25,26,27,28,29   /

            SETBANK INITBANK

            lda #0
            sta FrameAddress+1

            lda ObjFrame,y              ; animation frame (absolute)
            asl
            adc ObjFrame,y              ; Frametable is bank, AddressLO, AddressHI
            tax

            SETBANK BANK_FRAMETABLE

            lda FrameTable+1,x
            sta FrameAddress
            lda FrameTable+2,x
            sta FrameAddress+1

            lda FrameTable,x
            sta BANK

            ldx #0
            lda AFacing,y
            bpl nMirror
            ldx #BIG_ROWS*BIG_COLS
nMirror


    ; Now we have a pointer to the frame definition block (FrameAddress)
    ; Go through the entire BIG_ROWS*BIG_COLS entries and convert the matrix 16-sprite
    ; number into an absolute address pointing to the sprite data.  This will correspond
    ; nicely to the vars used for the actual draw.

            ldy #0
            lda (FrameAddress),y
            sta COLUP0
            sta COLUP1                  ; tentative colour

Mirrorable  iny
            sty Temp
            
            stx Temp+1
            lda BaseMirror,x
            tax

            lda (FrameAddress),y
            tay

            asl
            asl
            asl
            asl
            sta SPBASE,x

            tya
            lsr
            lsr
            lsr
            lsr
            clc
            adc #$F0
            sta SPBASE+1,x

            ldx Temp+1
            inx

            ldy Temp
            cpy #BIG_ROWS*BIG_COLS
            bcc Mirrorable


    ; That was really neat... all rows in one whack!
    ; There's one remaining byte in the frame definition, but currenty unused
    ; Now we have all the matrix pointers setup, and the bank... so we can draw!

            sta WSYNC

            ldy Player

    ; We want to reverse the order of the columns if player is facing left
    ; This is effectively pointing each of the column starting addresses into the
    ; data in reversed order (R-L instead of L-R).

            lda AFacing,y
            and #D7
            lsr
            lsr
            lsr
            lsr
            sta REFP0
            sta REFP1                   ;@17

            lda BGColour
            sta COLUBK


_RTS14      nop
            rts                         ; 14-cycle waste

    ;------------------------------------------------------------------------


NewFrame

            lda #$02
            sta WSYNC

            STA VBLANK          

            sta VSYNC           
            sta WSYNC           
            sta WSYNC
            sta WSYNC

            lda #$00
            sta VSYNC           

            lda #43             
            sta TIM64T

            lda #$00
            sta HMCLR

            jsr VBProcessing        ; spend 2000+ cycles


#if DEBUG
            lda INTIM
            bne Whoops
            inc BGColour            ; Overflow
Whoops
#endif



VblankLoop  lda INTIM
            bne VblankLoop      
            
            sta WSYNC           
            sta VBLANK          

    ;------------------------------------------------------------------------------
    ; START OF DISPLAY


            lda #150                ; NTSC
            sta TIM64T


    ;?
            lda     #%00010101      ; reflect, priority, 2 pixel wide ball
            sta     CTRLPF
    ;^?

            lda #PFCOL
            sta COLUPF
            
            lda #0
            sta PF0
            sta PF1
            sta PF2

            sta GRP0
            sta GRP1

    ;--------------------------------------------------------------------------
    ; Init the loop counters (16 rows each) for each of the rows

            lda #15
.ROW        SET 0
    REPEAT BIG_ROWS
            sta LoopCount + .ROW
.ROW        SET .ROW + 1
    REPEND


    ;--------------------------------------------------------------------------
    ; Centerpoint Adjustment based on frame

            ldy Player

            lda ObjFrame,y
            bpl NormalDraw
            jmp SkipDraw

NormalDraw  asl
            tax

            lda AFacing,y
            bpl nM

            sec
            lda DrawX
            sbc #48
            adc OffsetXY,x
            sta DrawX

            jmp doY

nM          sec
            lda DrawX
            sbc OffsetXY,x
            sta DrawX

doY         sec
            lda DrawY
            sbc OffsetXY+1,x
            sta DrawY
               

            jsr PositionObject
            jsr DRAWLINE0


SkipDraw    lda #0
            sta GRP0
            sta GRP1
            sta GRP0            ; buffered??
            sta GRP1            ; buffered??

            sta VDELP0
            sta VDELP1

            sta REFP0
            sta REFP1

            sta WSYNC



PadScreen   lda INTIM
            bne PadScreen

        ; Let's attempt a scoreline

            lda #70                ; NTSC
            sta TIM64T

            lda #0
            sta DrawY

            ldy Player
            lda ScorePos,y
            sta DrawX

            SETBANK BANK_FRAMETABLE

            lda FrameTable+1
            sta FrameAddress
            lda FrameTable+2
            sta FrameAddress+1

            lda FrameTable
            sta BANK                    ; now have correct sprites for alphas

            lda #15
            sta LoopCount+5


            lda Player
            asl 
            asl
            asl
            sec
            sbc Player
            sbc Player
            sta Temp

;            inc BaseAlpha

            lda BaseAlpha
            lsr
            lsr
            lsr
            and #15
            clc
            adc Temp
            tay
            iny


            lda #5
            sta Temp
            ldx #60

FillDigit   
            jsr FixSprite
            iny
            inx
            inx

            dec Temp
            bpl FillDigit

            sta WSYNC

            jsr PositionObject
            jsr DRAWLINE5

            lda #0
            sta GRP0
            sta GRP1
            sta GRP0            ; buffered??
            sta GRP1            ; buffered??

            sta VDELP0
            sta VDELP1

            sta WSYNC

PadScreen2  lda INTIM
            bne PadScreen2




;            sta WSYNC
;            sta WSYNC
;            sta WSYNC
;            sta WSYNC
;            sta WSYNC
;            sta WSYNC

            lda #D6+D1
            sta VBLANK                      ; end of screen - enter blanking
      
            sta HMCLR
            sta WSYNC

OverscanStart

            lda #25 ;35             
            sta TIM64T


            SETBANK BANK_STATE

            ldx Player
            jsr ProcessFSM                  ; process creature FSM logic

            SETBANK INITBANK

            jsr AnimateObject               ; process object animation sequence


;                lda     SWCHB
;                and     #$01
;                bne     NoResetSwitch

Overscan    lda INTIM
            bne Overscan

;            sta WSYNC           

            jmp NewFrame

ScorePos    .byte 112-48,112



Reset       SETBANK INITBANK
            jmp Cart_Init                   ; init code is in bank 1!


FixSprite
            lda (FrameAddress),y

            asl
            asl
            asl
            asl
            sta SPBASE,x

            lda (FrameAddress),y
            lsr
            lsr
            lsr
            lsr
            clc
            adc #$F0
            sta SPBASE+1,x
            rts


    ;--------------------------------------------------------------------------
    ; Object X,Y positioning
    ; Timing is absolutely critical here!

PositionObject

    ; Waste scanlines to start of object position, giving our vertical movement

            ldx DrawY
            inx
DelayVert   sta WSYNC
            dex
            bne DelayVert

    ; Now the horizontal magic

            ldx DrawX
            clc
            lda MoveTable,x
            sta HMP0
            adc #$10
            sta HMP1

            lda #1
            sta VDELP0
            sta VDELP1

            lda #3
            sta NUSIZ0
            sta NUSIZ1

            sta WSYNC

            lda #>JNDelayPos
            sta DelayPTR+1
            lda DelayTable,x
            sta DelayPTR
            jmp (DelayPTR)

            ALIGN   256

JNDelayPos  
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c5

            nop

            sta RESP0
            sta RESP1
            sta WSYNC
            sta HMOVE

            lda #>JNDelayDraw
            sta DelayPTR+1
            lda #<JNDelayDraw
            clc
            adc DelayTable,x
            sta DelayPTR

            sta WSYNC

            jsr _RTS14
            nop
            nop
            nop
            nop

            jmp (DelayPTR)

JNDelayDraw

            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c5

            nop
            rts

    ;--------------------------------------------------------------------------
    ; The beautiful, beautiful, draw loop which draws the entire matrix onscreen
    ; This caters for an adjustable (at compile-time) number of rows and also
    ; facilitates double-size pixels.

ROW         SET 0
            REPEAT BIG_ROWS
            SUBROUTINE

.S1         SET SPBASE + ( ROW * 12 )
.S2         SET SPBASE + ( ROW * 12 ) + 2
.S3         SET SPBASE + ( ROW * 12 ) + 4
.S4         SET SPBASE + ( ROW * 12 ) + 6
.S5         SET SPBASE + ( ROW * 12 ) + 8
.S6         SET SPBASE + ( ROW * 12 ) + 10


#if ROW = 0
DRAWLINE0
#endif
#if ROW = 1
DRAWLINE1
#endif
#if ROW = 2
DRAWLINE2
#endif
#if ROW = 3
DRAWLINE3
#endif
#if ROW = 4
DRAWLINE4
#endif
#if ROW = 5
DRAWLINE5
#endif

            jmp .P1             ; 3                 CONTINUE CODE

.TEMP       SET ROW/3
#if ( ROW = .TEMP*3 )
            ALIGN 256
#endif

.LAB1       nop                 ; 2                 ->starts at 76 on iteration 2+
.P1         nop                 ; 2
            nop                 ; 2

            ldy LoopCount + ROW ; 3

            lda (.S1),y         ; 5
            sta GRP0            ; 3
            lda (.S2),y         ; 5
            sta GRP1            ; 3
            lda (.S3),y         ; 5
            sta GRP0            ; 3

            lda (.S6),y         ; 5
            sta Temp            ; 3
            lda (.S5),y         ; 5                 Can use LAX (x),y  for optimisations here...
            tax                 ; 2
            lda (.S4),y         ; 5
            ldy Temp            ; 3
            sta GRP1            ; 3
            stx GRP0            ; 3
            sty GRP1            ; 3
            sta GRP0            ; 3
            
#if DOUBLESIZE

            jsr _RTS14          ; 14

                ; Beginning line 2 @-6

            ldy LoopCount + ROW ; 3

            lda (.S1),y         ; 5
            sta GRP0            ; 3
            lda (.S2),y         ; 5
            sta GRP1            ; 3
            lda (.S3),y         ; 5
            sta GRP0            ; 3

            nop                 ; 2
            nop                 ; 2
            nop                 ; 2
            nop                 ; 2

            lda (.S5),y         ; 5                 Can use LAX (x),y  for optimisations here...
            tax                 ; 2
            lda (.S4),y         ; 5
            ldy Temp            ; 3
            sta GRP1            ; 3
            stx GRP0            ; 3
            sty GRP1            ; 3
            sta GRP0            ; 3

#endif
            dec LoopCount + ROW ; 5
            bpl .LAB1           ; 3(2)  -> @76

ROW         SET ROW + 1
            REPEND
            
            rts


    ;--------------------------------------------------------------------------

            ALIGN 256

DelayTable
            .byte   37, 37
Q           SET     36
            REPEAT  37
            .byte   Q, Q, Q
Q           SET     Q-1
            REPEND

MoveTable
           .byte   $30, $20
            REPEAT  37
                .byte   $40, $30, $20
            REPEND




    ;--------------------------------------------------------------------------

; Player sprites are in N groups of 16 lines each, each being 6 sprites wide.
; Each of the 6 sprites vectors to a sprite number in the bank in which the frame
; is held.  Thus, each frame *MUST* fit within a bank.  The sprite number is
; multiplied by 16 and added to the bank base address to give an absolute
; address of the sprite data.

;     COL 0  COL 1  COL 2  COL 3  COL 4  COL 5
;   +------+------+------+------+------+------+                     \
;   |+0    |+2    |+4    |+6    |+8    |+10   |                      | 16 lines
;   |      |      |      |      |      |      |     ROW 0            |
;   |      |      |      |      |      |      |                     /
;   +------+------+------+------+------+------+                     \
;   |+12   |+14   |...   |      |      |      |                      | 16 lines
;   |      |      |      |      |      |      |     ROW 1            |
;   |      |      |      |      |      |      |                     /
;   +------+------+------+------+------+------+
;   (      (      (      (      (      (      (
;    )      )      )      )      )      )      ) 
;   +------+------+------+------+------+------+                     \
;   |      |      |      |      |      |      |                      | 16 lines
;   |      |      |      |      |      |      |     ...ROW n         |
;   |      |      |      |      |      |      |     (BIG_ROWS-1)    /
;   +------+------+------+------+------+------+

; Diagram shows a complete 'large sprite' consisting of n rows.
; The +0..+n inside the boxes represent the SPBASE offset of the zero page (),y pointer
; to the sprite as setup by the draw code.  This is internal but shown here for clarity.


    ;--------------------------------------------------------------------------
    ; SAMPLE FRAME DEFINITION
    ; Format is...
    ;   colour      colour of frame (tentative only)
    ;   x * y       matrix of sprite numbers in the bank (each being an index to a 16-sprite)
    ;               since the bank is 2K, there's 2K/16 max sprites = max 128
 




BaseMirror

.SP         SET 0
            REPEAT BIG_ROWS*BIG_COLS
            .byte .SP
.SP         SET .SP + 2
            REPEND
.SPBASE     SET 0
            REPEAT BIG_ROWS
.SPOFF      SET 10
            REPEAT BIG_COLS
            .byte .SPBASE + .SPOFF
.SPOFF      SET .SPOFF-2
            REPEND
.SPBASE     SET .SPBASE + 12
            REPEND



OffsetXY

    .byte 0,0 ; alphanum

    .byte 17,92 ;0
    .byte 16,92 ;1
    .byte 15,91 ;2
    .byte 17,92 ;3
    .byte 20,91 ;4
    .byte 18,91 ;5
    .byte 17,90 ; baby7
    .byte 17,93
    .byte 19,92
    .byte 14,91
    .byte 16,91
    .byte 23,92
    .byte 22,92
    .byte 19,90
    .byte 22,92
    .byte 22,92
    .byte 22,93
    .byte 23,91
    .byte 22,94
    .byte 20,93
    .byte 27,93
    .byte 28,92
    .byte 19,93 ; baby23
    .byte 12,91
    .byte 18,91
    .byte 16,91
    .byte 14,91
    .byte 13,91 ;baby28
    .byte 14,91
    .byte 13,92
    .byte 13,91
    .byte 15,92
    .byte 15,92 ;baby33
    .byte 17,93
    .byte 21,90
    .byte 25,90
    .byte 19,89
    .byte 27,89
    .byte 25,90
    .byte 23,91
    .byte 27,89
    .byte 22,89
    .byte 21,88 ;baby43
    .byte 24,90
    .byte 19,93
    .byte 14,93
    .byte 18,94
    .byte 13,91
    .byte 14,92
    .byte 14,92
    .byte 14,89
    .byte 14,90
    .byte 16,91
    .byte 19,92
    .byte 14,90
    .byte 13,91
    .byte 26,92 ;57
    .byte 28,92
    .byte 23,92
    .byte 22,92
    .byte 20,92
    .byte 23,90
    .byte 22,91
    .byte 20,90
    .byte 22,91
    .byte 20,92
    .byte 24,93
    .byte 19,93 ;baby68
    .byte 15,91
    .byte 15,91
    .byte 18,93
    .byte 17,91







/*T16HI
.SP         SET BANKPAGE
            REPEAT 128
            .byte >.SP
.SP         SET .SP + 16
            REPEND
*/
    ;--------------------------------------------------------------------------

            ORG $7ffa   ;IntVectors

            .word   Reset           ; NMI
            .word   Reset           ; RESET
            .word   Reset           ; IRQ

    		END

